import requests


class Session(requests.Session):

    def _store_xsrf_token(self, response):
        xsrf_token = response.cookies.get('XSRF-TOKEN')
        if xsrf_token:
            self.xsrf_token = xsrf_token

    def _set_xsrf_headers(self, kwargs):
        xsrf_header = {
            'X-XSRF-TOKEN': self.xsrf_token,
        }
        if 'headers' in kwargs:
            kwargs['headers'].update(xsrf_header)
        else:
            kwargs['headers'] = xsrf_header
        return kwargs

    def get(self, *args, **kwargs):
        response = super(Session, self).get(*args, **kwargs)
        self._store_xsrf_token(response)
        return response

    def post(self, *args, **kwargs):
        if kwargs.pop('add_xsrf_token', True):
            kwargs = self._set_xsrf_headers(kwargs)
        response = super(Session, self).post(*args, **kwargs)
        self._store_xsrf_token(response)
        return response

    def put(self, *args, **kwargs):
        if kwargs.pop('add_xsrf_token', True):
            kwargs = self._set_xsrf_headers(kwargs)
        response = super(Session, self).put(*args, **kwargs)
        self._store_xsrf_token(response)
        return response

    def delete(self, *args, **kwargs):
        kwargs = self._set_xsrf_headers(kwargs)
        response = super(Session, self).delete(*args, **kwargs)
        self._store_xsrf_token(response)
        return response



# -*- coding: utf-8 -*-
import re
import os
from collections import OrderedDict

import json
import six
import requests
from six.moves.http_cookiejar import LWPCookieJar

import neres.urls as urls
import neres.session as session

session = session.Session()


def initialize_cookiejar(cookiejar):
    session.cookies = LWPCookieJar(cookiejar)
    if not os.path.exists(cookiejar):
        session.cookies.save()
        os.chmod(cookiejar, 0o600)
    else:
        session.cookies.load(ignore_discard=True)


def check_if_logged_in():
    response = session.get(urls.IDLE, allow_redirects=False)
    response.raise_for_status()
    if response.status_code == 200:
        return True
    return False


def login(email, password):
    session.cookies.clear()
    response = session.get(urls.LOGIN)
    response.raise_for_status()

    try:
        token = re.search(b'name="authenticity_token" value="(.+?)"',
                          response.content).groups()[0]
    except AttributeError:
        Exception("Can't get CSRF token to login.")

    payload = {
        'login[email]': email,
        'login[password]': password,
        'login[remember_me]': '1',
        'return_to': 'https://rpm.newrelic.com/auth/newrelic',
        'utf8': '✓',
        'authenticity_token': token,
        'commit': 'Sign in'
    }
    response = session.post(urls.LOGIN, data=payload, add_xsrf_token=False)
    response.raise_for_status()
    if b'login_email' in response.content:
        raise Exception('Login Failed')
    session.cookies.save(ignore_discard=True)


def get_monitors(account):
    offset = 0
    monitors = []

    while True:
        url = urls.MONITORS_V2.format(account=account, offset=offset)
        response = session.get(url)
        response.raise_for_status()
        data = response.json().get('data')
        if not data:
            break
        monitors.extend(data)
        offset += 15

    for monitor in monitors:
        # Fetch monitor stoplight
        url = urls.MONITOR_STOPLIGHT.format(account=account, monitor=monitor['id'])
        response = session.get(url)
        response.raise_for_status()
        monitor.update(response.json())

        # Fetch monitor locations.
        monitor.update(get_monitor(account, monitor['id']))

    # sort data by name
    monitors = sorted(monitors, key=lambda x: x['name'])

    return monitors


def delete_monitor(account, monitor):
    url = urls.MONITOR_JSON.format(account=account, monitor=monitor)
    response = session.get(url)
    response.raise_for_status()
    headers = {
        'Referer': response.url,
    }

    url = urls.MONITOR_JSON.format(account=account, monitor=monitor)
    response = session.delete(url, headers=headers)
    response.raise_for_status()


def get_locations(account):
    url = urls.MONITOR_LOCATIONS.format(account=account)
    response = session.get(url)
    response.raise_for_status()

    return response.json()


def get_monitor(account, monitor):
    url = urls.MONITOR_JSON.format(account=account, monitor=monitor)
    response = session.get(url)
    response.raise_for_status()

    data = response.json()

    # get stoplight status
    url = urls.MONITOR_STOPLIGHT.format(account=account, monitor=monitor)
    response = session.get(url)
    response.raise_for_status()

    data.update(response.json())
    return data


def update_monitor(account, monitor, *args, **kwargs):
    data = get_monitor(account, monitor)
    data['id'] = monitor
    data.pop('createdAt', None)
    data.pop('modifiedAt', None)

    data['currentChecks'] = 43200 / data['frequency']
    data['slaThreshold'] = int(data['slaThreshold'])

    for prop in ['name', 'uri', 'frequency',
                 'emails', 'locations', 'slaThreshold', 'status']:
        value = kwargs.get(prop, None)
        if value:
            data[prop] = value

    for prop in ['emails', 'locations']:
        if kwargs.get('clear_{}'.format(prop)):
            data[prop] = []

        values = kwargs.get('add_{}'.format(prop))
        if values:
            if isinstance(values, six.string_types):
                values = [values]
            data[prop] = list(set(data[prop] + list(values)))

        values = kwargs.get('remove_{}'.format(prop))
        if values:
            if isinstance(values, six.string_types):
                prop = [values]
            data[prop] = list(set(data[prop]) - set(values))

    _construct_metadata(
        validation_string=kwargs.get('validation_string'),
        bypass_head_request=kwargs.get('bypass_head_request'),
        verify_ssl=kwargs.get('verify_ssl'),
        redirect_is_failure=kwargs.get('redirect_is_failure'),
        metadata=data['metadata'])

    headers = {
        'Referer': urls.MONITOR.format(account=account, monitor=monitor),
        'Content-Type': 'application/json;charset=utf-8',
    }

    url = urls.MONITOR_JSON.format(account=account, monitor=monitor)
    response = session.put(url, data=json.dumps(data), headers=headers)

    try:
        response.raise_for_status()
    except requests.exceptions.HTTPError:
        try:
            error = response.json()['error']
        except (ValueError, KeyError):
            error = 'Bad request'
        return (1, error, {})

    url = urls.MONITOR.format(account=account, monitor=response.json()['id'])
    return (0, url, response.json())


def _construct_metadata(validation_string=None, bypass_head_request=None,
                        verify_ssl=None, redirect_is_failure=None,
                        metadata=None):
    if metadata is None:
        metadata = {}

    if bypass_head_request is True:
        metadata['nr.synthetics.metadata.job.options.simple.bypass.head'] = True
    elif bypass_head_request is False:
        metadata.pop('nr.synthetics.metadata.job.options.simple.bypass.head', None)

    if validation_string:
        metadata['nr.synthetics.metadata.job.options.response-validation'] = validation_string
    elif validation_string is False:
        metadata.pop('nr.synthetics.metadata.job.options.response-validation', None)

    if verify_ssl is True:
        metadata['nr.synthetics.metadata.job.options.tlsValidation'] = True
        metadata['nr.synthetics.monitor.tls-validation'] = True
    elif verify_ssl is False:
        metadata.pop('nr.synthetics.metadata.job.options.tlsValidation', None)
        metadata.pop('nr.synthetics.monitor.tls-validation', None)

    if redirect_is_failure is True:
        metadata['nr.synthetics.metadata.job.options.redirectIsFailure'] = True
        metadata['nr.synthetics.metadata.job.options.simple.redirect.is.failure'] = True
    elif redirect_is_failure is False:
        metadata.pop('nr.synthetics.metadata.job.options.redirectIsFailure', None)
        metadata.pop('nr.synthetics.metadata.job.options.simple.redirect.is.failure', None)

    return metadata


def create_monitor(account, name, uri, frequency, locations, emails=[],
                   validation_string='', bypass_head_request=False,
                   verify_ssl=False, redirect_is_failure=False,
                   slaThreshold=7):
    if isinstance(locations, six.string_types):
        locations = [locations]

    if isinstance(emails, six.string_types):
        emails = [emails]

    if validation_string and not bypass_head_request:
        raise Exception('Response validation requires to bypass HEAD request.')

    response = session.get(urls.NEW_MONITOR.format(account=account))
    response.raise_for_status()
    headers = {
        'Referer': response.url,
        'Content-Type': 'application/json;charset=utf-8',
    }

    metadata = _construct_metadata(validation_string, bypass_head_request,
                                   verify_ssl, redirect_is_failure)
    data = {
        'accountId': account,
        'name': name,
        'type': 'SIMPLE',
        'frequency': frequency,
        'uri': uri,
        'status': 'ENABLED',
        'slaThreshold': slaThreshold,
        'locations': locations,
        'conditions': [],
        'metadata': metadata,
        'emails': emails,
    }
    data = json.dumps(data)
    url = urls.MONITORS.format(account=account)
    response = session.post(url, data=data, headers=headers)

    try:
        response.raise_for_status()
    except requests.exceptions.HTTPError:
        try:
            error = response.json()['error']
        except (ValueError, KeyError):
            error = 'Bad request'
        return (1, error, {})

    url = urls.MONITOR.format(account=account, monitor=response.json()['id'])
    return (0, url, response.json())


def get_accounts():
    response = session.get(urls.SYNTHETICS)
    response.raise_for_status()

    accountId = re.search(b'"accountId":(\d+)', response.content).groups()[0]

    response = session.get(urls.ACCOUNT_INFO.format(account=accountId.decode('utf-8')))
    response.raise_for_status()

    return response.json().get('accountList')


def get_state(account):
    data = []
    monitors = get_monitors(account)

    for monitor in monitors:
        monitor_details = get_monitor(account, monitor['id'])

        monitor_data = OrderedDict([
            ('id', monitor['id']),
            ('name', monitor_details['name']),
            ('status', monitor_details['status']),
            ('uri', monitor_details['uri']),
            ('slaThreshold', monitor_details['slaThreshold']),
            ('emails', monitor_details['emails'] or ''),
            ('locations', monitor_details['locations'] or ''),
            ('frequency', monitor_details['frequency']),
            ('verify_ssl', False),
            ('validation_string', False),
            ('bypass_head_request', False),
            ('redirect_is_failure', False),
        ])

        if monitor_details['metadata']:
            m = monitor_details['metadata']

            if m.get('nr.synthetics.metadata.job.options.simple.bypass.head') == 'true':
                monitor_data['bypass_head_request'] = True

            response_validation = m.get(
                'nr.synthetics.metadata.job.options.response-validation', False)
            monitor_data['validation_string'] = response_validation

            if m.get('nr.synthetics.monitor.tls-validation'):
                monitor_data['verify_ssl'] = True

            if m.get('nr.synthetics.metadata.job.options.simple.redirect.is.failure'):
                monitor_data['redirect_is_failure'] = True
        data.append(monitor_data)

    return data



# -*- coding: utf-8 -*-
# Spinner based on https://gist.github.com/cevaris/79700649f0543584009e
from __future__ import print_function

import itertools
import sys
import time
import threading


class Spinner(object):
    spinner_cycle = itertools.cycle(['⣾', '⣷', '⣯', '⣟', '⡿', '⢿', '⣻', '⣽'])

    def __init__(self, message='', remove_message=True):
        self.stop_running = threading.Event()
        self.spin_thread = threading.Thread(target=self.init_spin)
        self.message = message
        self.remove_message = remove_message

    def start(self):
        self.spin_thread.start()

    def stop(self):
        self.stop_running.set()
        if self.message and self.remove_message:
            sys.stderr.write('\b' * len(self.message))
        self.spin_thread.join()

    def init_spin(self):
        while not self.stop_running.is_set():
            sys.stderr.write(next(self.spinner_cycle))
            sys.stderr.flush()
            time.sleep(0.35)
            sys.stderr.write('\b')

    def __enter__(self):
        if self.message:
            sys.stderr.write(self.message)
        self.start()

    def __exit__(self, *args):
        self.stop()



SYNTHETICS = 'https://synthetics.newrelic.com'
LOGIN = 'https://login.newrelic.com/login'
MONITORS = 'https://synthetics.newrelic.com/accounts/{account}/monitors.json'
MONITORS_V2 = ('https://synthetics.newrelic.com/accounts/{account}'
               '/v2/monitors.json?offset={offset}&limit=15')
ACCOUNT_INFO = 'https://synthetics.newrelic.com/accounts/{account}/info.json'
MONITOR_JSON = 'https://synthetics.newrelic.com/accounts/{account}/monitors/{monitor}.json'
MONITOR = 'https://synthetics.newrelic.com/accounts/{account}/monitors/{monitor}'
NEW_MONITOR = 'https://synthetics.newrelic.com/accounts/{account}/monitors/new'
MONITOR_LOCATIONS = 'https://synthetics.newrelic.com/accounts/{account}/locations/list.json'
IDLE = 'https://login.newrelic.com/idle_timeout'
MONITOR_STOPLIGHT = ('https://synthetics.newrelic.com/accounts/{account}'
                     '/monitors/{monitor}/stoplight.json')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
import sys
import os
import platform
import subprocess
from datetime import datetime

import click
import humanize
import yaml
import yamlordereddictloader
from terminaltables import SingleTable

import neres.newrelic as newrelic
import neres.urls as urls
from .spinner import Spinner


@click.command(help='Add a new monitor')
@click.argument('name')
@click.argument('uri')
@click.option('--location', default=['AWS_US_WEST_1'], multiple=True,
              help='Monitor locations. Defaults to AWS_US_WEST_1. Get a list of available monitor'
                   'locations with `list-locations`. Repeat for multiple locations.')
@click.option('--frequency', default='10',
              type=click.Choice(['1', '5', '10', '15', '30', '60', '360', '720', '1440']),
              callback=lambda x, y, z: int(z),
              help='Monitor frequency. Defaults to 10 minutes')
@click.option('--email', default=[], multiple=True,
              help='Send alerts to email. Repeat for multiple alerts.')
@click.option('--validation-string', default='',
              help='Add a validation string to look for in the response (optional)')
@click.option('--bypass-head-request', default=False, is_flag=True,
              help=('Send full HTTP GET requests. '
                    'Will be automatically set if validation-string is provided.'))
@click.option('--verify-ssl', default=False, is_flag=True,
              help='Run a detailed OpenSSL handshake and alert if it fails.')
@click.option('--redirect-is-failure', default=False, is_flag=True,
              help='HTTP redirect codes result in a failure')
@click.option('--sla-threshold', default=7, type=int,
              help='Set Apdex Threshold. Defaults to 7 seconds.')
@click.option('--raw', default=False, is_flag=True, help='Return raw json response')
@click.pass_context
def add_monitor(ctx, name, uri, location, frequency, email, validation_string,
                bypass_head_request, verify_ssl, redirect_is_failure, sla_threshold, raw):
    if validation_string:
        # We must bypass head request we're to validate string.
        bypass_head_request = True

    with Spinner('Creating monitor: ', remove_message=raw):
        status, message, monitor = newrelic.create_monitor(
            ctx.obj['ACCOUNT'], name, uri, frequency, location, email,
            validation_string, bypass_head_request, verify_ssl, redirect_is_failure,
            sla_threshold)

    if raw:
        print(json.dumps(monitor))
        return

    if status == 0:
        print(click.style(u'OK', fg='green', bold=True))
        print('Monitor: ' + message)
    else:
        print(click.style(u'Error', fg='red', bold=True))
        raise click.ClickException(message)


@click.command(help='Update an existing monitor')
@click.argument('monitor')
@click.option('--name', default=None, help='Change the name of the monitor')
@click.option('--uri', default=None, help='Change the URI to monitor')
@click.option('--add-location', 'add_locations', default=None, multiple=True,
              help=('Add a monitor location. Repeat for multiple locations. '
                    'Get available locations with `list-locations`'))
@click.option('--clear-locations', default=False, is_flag=True,
              help='Remove all monitor locations. Must be combined with `add-location`')
@click.option('--remove-location', 'remove_locations', default=None, multiple=True,
              help='Remove a monitor location. Repeats for multiple locations')
@click.option('--add-email', 'add_emails', default=None, multiple=True,
              help='Add an email to receive alerts. Repeat for multiple emails')
@click.option('--remove-email', 'remove_emails', default=None, multiple=True,
              help='Remove email from alert list. Repeat for multiple emails')
@click.option('--clear-emails', default=False, is_flag=True,
              help='Remove all emails from alert list')
@click.option('--frequency', default=None,
              type=click.Choice(['1', '5', '10', '15', '30', '60', '360', '720', '1440']),
              help='Change monitor frequency.')
@click.option('--sla-threshold', 'slaThreshold', default=None,
              help='Change Apdex Threshold')
@click.option('--validation-string', default=None, help='Set or change the validation string')
@click.option('--no-validation-string', default=None, is_flag=True,
              help='Remove validation string check')
@click.option('--bypass-head-request/--no-bypass-head-request', default=None, is_flag=True,
              help='Set / unset sending of full HTTP GET requests')
@click.option('--verify-ssl/--no-verify-ssl', default=None, is_flag=True,
              help='Set / unset OpenSSL verification')
@click.option('--redirect-is-failure/--no-redirect-is-failure', default=None, is_flag=True,
              help='Set / unset redirect is failure check')
@click.option('--status',
              type=click.Choice(['enabled', 'disabled', 'muted', 'ENABLED', 'DISABLED', 'MUTED']),
              default=None, help='Set monitor status',
              callback=lambda x, y, z: z.upper() if z else None)  # Yes this is horrible.
@click.option('--raw', default=False, is_flag=True, help='Return raw json response')
@click.pass_context
def update_monitor(ctx, monitor, **kwargs):
    if kwargs['no_validation_string']:
        if kwargs['validation_string']:
            raise click.ClickException(
                'Flags --validation-string and --no-validation-string cannot be combined')

        kwargs['validation_string'] = False

    if kwargs['validation_string']:
        # We must bypass head request we're to validate string.
        kwargs['bypass_head_request'] = True

    if kwargs['clear_locations'] and not kwargs['add_locations']:
        raise click.ClickException(
            'You need at least one location. Combine --clear-locations'
            'with --add-location.')

    with Spinner('Updating monitor: ', remove_message=kwargs['raw']):
        status, message, monitor = newrelic.update_monitor(ctx.obj['ACCOUNT'], monitor, **kwargs)

    if kwargs['raw']:
        print(json.dumps(monitor))
        return

    if status == 0:
        print(click.style(u'OK', fg='green', bold=True))
        print('Monitor: ' + message)
    else:
        print(click.style(u'Error', fg='red', bold=True))
        raise click.ClickException(message)


@click.command(help='Get information for a monitor')
@click.argument('monitor')
@click.option('--raw', default=False, is_flag=True, help='Return raw json response')
@click.pass_context
def get_monitor(ctx, monitor, raw):
    with Spinner('Fetching monitor: '):
        monitor = newrelic.get_monitor(ctx.obj['ACCOUNT'], monitor)

    if raw:
        print(json.dumps(monitor))
        return

    severity = monitor.get('severity', 0)
    if severity == 2:
        health = click.style(u'✔', fg='green')
    elif severity == 1:
        health = click.style(u'❢', fg='yellow')
    else:
        health = click.style(u'✖', fg='red')
    monitor['health'] = health

    status = monitor['status'].lower()
    if status in ('muted', 'disabled'):
        status = click.style(u'❢ {}'.format(status), fg='yellow')
    else:
        status = click.style(u'✔ OK', fg='green')

    data = [
        ['Monitor', monitor['id']],
        ['Status', status],
        ['Health', health],
        ['Name', monitor['name']],
        ['URI', monitor['uri']],
        ['Type', monitor['type']],
        ['Locations', ', '.join(monitor['locations'])],
        ['slaThreshold', monitor['slaThreshold']],
        ['Emails', ', '.join(monitor['emails'])],
        ['Frequency', monitor['frequency']],
        ['Created', monitor['createdAt']],
        ['Modified', monitor['modifiedAt']],

    ]

    table = SingleTable(data)
    table.title = click.style('Monitor', fg='black')
    print(table.table)


@click.command(help='Delete a monitor')
@click.argument('monitor')
@click.option('--confirm', default=None, help='Skip confirmation prompt by supplying monitor id')
@click.pass_context
def delete_monitor(ctx, monitor, confirm):
    if not confirm:
        confirm = click.prompt('''
 ! WARNING: Destructive Action
 ! This command will destroy the monitor: {monitor}
 ! To proceed, type "{monitor}" or
   re-run this command with --confirm={monitor}

'''.format(monitor=monitor), prompt_suffix='> ')
    if confirm.strip() != monitor:
        print('abort')
        sys.exit(1)

    with Spinner('Deleting monitor {}: '.format(monitor), remove_message=False):
        newrelic.delete_monitor(ctx.obj['ACCOUNT'], monitor)
    print(click.style(u'OK', fg='green', bold=True))


@click.command(help='List available monitor locations')
@click.option('--raw', default=False, is_flag=True, help='Return raw json response')
@click.pass_context
def list_locations(ctx, raw):
    with Spinner('Fetching locations: '):
        locations = newrelic.get_locations(ctx.obj['ACCOUNT'])

    if raw:
        print(json.dumps(locations))
        return

    data = [[
        '#',
        'City',
        'Continent',
        'Code',
        'Availability',
        'Accessibility',
    ]]

    for number, location in enumerate(locations.values()):
        available = click.style(u'✔', fg='green')
        if not location['available']:
            click.style(u'✖', fg='red')

        private = 'Private' if location['private'] else 'Public'

        data.append([
            number,
            location['label'],
            location['continent'],
            location['name'],
            available,
            private,
        ])

    table = SingleTable(data)
    table.title = click.style('Locations', fg='black')

    for i in [0, 4, 5]:
        table.justify_columns[i] = 'right'

    print(table.table)


@click.command(help='List monitors')
@click.option('--ids-only', default=False, is_flag=True, help='List monitor IDs only')
@click.option('--raw', default=False, is_flag=True, help='Return raw json response')
@click.pass_context
def list_monitors(ctx, ids_only, raw):
    with Spinner('Fetching monitors: '):
        monitors = newrelic.get_monitors(ctx.obj['ACCOUNT'])

    if raw:
        print(json.dumps(monitors))
        return

    if ids_only:
        for monitor in monitors:
            print(monitor['id'])
        return

    data = [[
        '#',
        'H',
        'S',
        'Name',
        'ID',
        'Success\nRate',
        'Avg Size',
        'Load time\n(50th PR)',
        'Load time\n(95th PR)',
        'Frequency',
        'Loca\ntions',
        'Notif\nEmails',
    ]]

    for monitor in monitors:
        severity = monitor.get('severity', 0)
        if severity == 2:
            health = click.style(u'✔', fg='green')
        elif severity == 1:
            health = click.style(u'❢', fg='yellow')
        else:
            health = click.style(u'✖', fg='red')
        monitor['health'] = health

        status = monitor['status'].lower()
        if status in ('muted', 'disabled'):
            status = click.style(u'❢', fg='yellow')
        else:
            status = click.style(u'✔', fg='green')
        monitor['status'] = status

    for number, monitor in enumerate(monitors, 1):

        data.append([
            number,
            monitor['health'],
            monitor['status'],
            monitor['name'],
            monitor['id'],
            '{:.1f}%'.format(100 * monitor.get('success_ratio', 0)),
            humanize.naturalsize(monitor.get('avg_size', 0), binary=True),
            '{:.1f} ms'.format(monitor.get('load_time_50th_pr', 0)),
            '{:.1f} ms'.format(monitor.get('load_time_95th_pr', 0)),
            '{} min'.format(monitor['frequency']),
            len(monitor['locations']),
            len(monitor['emails'])
        ])

    table = SingleTable(data)
    table.title = click.style('Monitors', fg='black')

    for i in [1, 2]:
        table.justify_columns[i] = 'center'

    for i in [0, 5, 6, 7, 8, 9, 10, 11]:
        table.justify_columns[i] = 'right'

    table.justify_columns[3] = 'left'

    print(table.table)


@click.command(help='Open monitor in Web browser')
@click.argument('monitor')
@click.pass_context
def open_monitor(ctx, monitor):
    url = urls.MONITOR.format(account=ctx.obj['ACCOUNT'], monitor=monitor)
    if platform.system() == 'Windows':
        os.startfile(url)
    elif platform.system() == 'Darwin':
        subprocess.Popen(['open', url])
    else:
        subprocess.Popen(['xdg-open', url])


@click.command(help='List accounts')
@click.option('--raw', is_flag=True, default=False, help='Return raw json response')
def list_accounts(raw):
    with Spinner('Fetching accounts: '):
        accounts = newrelic.get_accounts()

    if raw:
        for account in accounts:
            print(account[0])

    data = [[
        'ID',
        'Name'
    ]]
    for account in accounts:
        data.append([account['id'], account['name']])

    table = SingleTable(data)
    table.title = click.style('Accounts', fg='black')
    print(table.table)


@click.command(help='Login to newrelic')
@click.pass_context
def login(ctx):
    email = ctx.obj['EMAIL']
    if not email:
        email = click.prompt('Email')

    password = ctx.obj['PASSWORD']
    if not password:
        password = click.prompt('Password', hide_input=True)

    with Spinner('Logging in: ', remove_message=False):
        newrelic.login(email, password)
    print(click.style(u'OK', fg='green', bold=True))


@click.command(help='Update from state')
@click.argument('statefile', type=click.File('rb'))
@click.option('--apply', default=False, is_flag=True)
@click.pass_context
def update_from_statefile(ctx, apply, statefile):
    if not apply:
        print('This is a dry run. Run with --apply to make the changes.\n')

    with Spinner('Getting current state: '):
        there_data = newrelic.get_state(ctx.obj['ACCOUNT'])
    here_data = yaml.load(statefile)

    changes = 0
    for monitor in here_data:
        for there_monitor in there_data:
            if there_monitor['id'] == monitor['id']:
                break
        else:
            print('Monitor {} only exists in statefile, skipping.'.format(monitor['id']))
            continue

        if there_monitor != monitor:
            monitor_id = monitor.pop('id')
            with Spinner('Updating monitor {}: '.format(monitor_id), remove_message=False):
                if apply:
                    status, message, _ = newrelic.update_monitor(ctx.obj['ACCOUNT'],
                                                                 monitor_id,
                                                                 **monitor)
                    if status == 0:
                        print(click.style(u'OK', fg='green', bold=True))
                    else:
                        print(click.style(u'Error', fg='red', bold=True))
                        raise click.ClickException(message)
            changes += 1

    if changes == 0:
        print('No changes made.')
    else:
        print('Successfully updated {} monitors'.format(changes))


@click.command(help='Get state')
@click.pass_context
def get_state(ctx, return_data=False):
    with Spinner('Fetching state: '):
        state = newrelic.get_state(ctx.obj['ACCOUNT'])

    print('# Generated on {}'.format(datetime.utcnow().isoformat()))
    print(yaml.dump(
        state,
        allow_unicode=True,
        default_flow_style=False,
        Dumper=yamlordereddictloader.SafeDumper,
    ))


@click.group()
@click.option('--email', help='New Relic login email')
@click.option('--password', help='New Relic login password')
@click.option('--account', help=('New Relic account to work on. You can get a list of accounts '
                                 'with `list-accounts`. Defaults to first listed account'))
@click.option('--environment', default='newrelic',
              help=('Default `newrelic`. Define different environments for '
                    'different New Relic accounts.'))
@click.pass_context
def cli(ctx, email, password, account, environment):
    cookiejar = os.path.expanduser('~/.config/neres/{}.cookies'.format(environment))
    if not os.path.exists(os.path.dirname(cookiejar)):
        os.makedirs(os.path.dirname(cookiejar), 0o700)
    newrelic.initialize_cookiejar(cookiejar)

    if ctx.invoked_subcommand != 'login':
        with Spinner('Authorizing: '):
            if all([email, password]):
                newrelic.login(email, password)
            else:
                if not newrelic.check_if_logged_in():
                    raise click.ClickException('Login first')

        if not account and ctx.invoked_subcommand != 'list-accounts':
            account = newrelic.get_accounts()[0]['id']

    ctx.obj = {}
    ctx.obj['ACCOUNT'] = account
    ctx.obj['EMAIL'] = email
    ctx.obj['PASSWORD'] = password


cli.add_command(list_monitors, name='list-monitors')
cli.add_command(list_locations, name='list-locations')
cli.add_command(delete_monitor, name='delete-monitor')
cli.add_command(get_monitor, name='get-monitor')
cli.add_command(add_monitor, name='add-monitor')
cli.add_command(update_monitor, name='update-monitor')
cli.add_command(list_accounts, name='list-accounts')
cli.add_command(open_monitor, name='open')
cli.add_command(login, name='login')
cli.add_command(get_state, name='get-state')
cli.add_command(update_from_statefile, name='update-from-statefile')


if __name__ == '__main__':
    cli(auto_envvar_prefix='NERES')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pip
from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()


requirements = [str(item.req) for item in
                pip.req.parse_requirements('requirements.txt',
                                           session=pip.download.PipSession())
                if item.req]

test_requirements = [
    str(item.req) for item in
    pip.req.parse_requirements('requirements_dev.txt',
                               session=pip.download.PipSession())
    if item.req]


setup(
    name='neres',
    version='0.4.3',
    description="(unofficial) NewRelic Synthetics CLI",
    long_description=readme + '\n\n' + history,
    author="Giorgos Logiotatidis",
    author_email='giorgos@sealabs.net',
    url='https://github.com/glogiotatidis/neres',
    packages=[
        'neres',
    ],
    package_dir={'neres':
                 'neres'},
    entry_points={
        'console_scripts': [
            'neres=neres.cli:cli'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license="GNU General Public License v3",
    zip_safe=False,
    keywords='neres',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements,
)
